@@ -25,6 +25,8 @@
 
 import dbus
 import os
+import gi
+gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk
 
 from Dell.recovery_gtk import DellRecoveryToolGTK, translate_widgets
