@@ -42,6 +42,8 @@ import syslog
 import glob
 import zipfile
 import tarfile
+import gi
+gi.require_version('UDisks', '2.0')
 from gi.repository import GLib, UDisks
 
 NAME = 'dell-bootstrap'
@@ -76,7 +78,7 @@ class PageNoninteractive(PluginUI):
     def __init__(self, controller, *args, **kwargs):
         self.controller = controller
         PluginUI.__init__(self, controller, *args, **kwargs)
-    
+
     def get_type(self):
         '''For the noninteractive frontend, get_type always returns an empty str
             This is because the noninteractive frontend always runs in "factory"
@@ -122,9 +124,10 @@ class PageGtk(PluginUI):
         oem = 'UBIQUITY_OEM_USER_CONFIG' in os.environ
 
         self.efi = False
-        self.genuine = magic.check_vendor() and not magic.check_msdm()
+        self.genuine = magic.check_vendor()
 
         if not oem:
+            gi.require_version('Gtk', '3.0')
             from gi.repository import Gtk
             builder = Gtk.Builder()
             builder.add_from_file('/usr/share/ubiquity/gtk/stepDellBootstrap.ui')
@@ -325,7 +328,7 @@ class PageGtk(PluginUI):
             return model.get_value(iterator, 0)
         else:
             return ""
- 
+
     def advanced_callback(self, widget, data = None):
         """Callback when an advanced widget is toggled"""
         if widget == self.proprietary_combobox:
@@ -367,15 +370,27 @@ class Page(Plugin):
             swap = item.get_swapspace()
             if not swap:
                 continue
+
             part = item.get_partition()
             if not part:
                 continue
+
             #Check if the swap is active or not
             swap_active = swap.get_cached_property("Active").get_boolean()
             if not swap_active:
                 continue
+
+            block = item.get_block()
+            if not block:
+                continue
+
+            device = block.get_cached_property('Device').get_bytestring().decode('utf-8')
+
             swap.call_stop_sync(no_options)
-            part.call_delete_sync(no_options)
+
+            # Only delete the swap partitions on the target
+            if device.startswith(self.device):
+                part.call_delete_sync(no_options)
 
     def sleep_network(self):
         """Requests the network be disabled for the duration of install to
@@ -385,7 +400,7 @@ class Page(Plugin):
         try:
             backend_iface = dbus.Interface(bus.get_object(magic.DBUS_BUS_NAME, '/RecoveryMedia'), magic.DBUS_INTERFACE_NAME)
             backend_iface.force_network(False)
-            backend_iface.request_exit() 
+            backend_iface.request_exit()
         except Exception:
             pass
 
@@ -400,7 +415,7 @@ class Page(Plugin):
     def clean_recipe(self):
         """Cleans up the recipe to remove swap if we have a small drive"""
 
-        #If we are in dynamic (dell-recovery/swap=dynamic) and small drive 
+        #If we are in dynamic (dell-recovery/swap=dynamic) and small drive
         #   or we explicitly disabled (dell-recovery/swap=false)
         if self.test_swap():
             self.log("Performing swap recipe fixup (%s, hdd: %i, mem: %f)" % \
@@ -414,13 +429,24 @@ class Page(Plugin):
 
     def remove_extra_partitions(self):
         """Removes partitions we are installing on for the process to start"""
-        #check for small disks.
-        #on small disks or big mem, don't look for extended or delete swap.
+       #check for small disks.
+       #on small disks or big mem, don't look for extended or delete swap.
         swap_part = EFI_SWAP_PARTITION
+        os_part = EFI_OS_PARTITION
+
+       # check dual boot or not
+        try:
+            if self.db.get('dell-recovery/dual_boot')=='true':
+           ##dual boot get the partition number of OS and swap
+                os_label = self.db.get('dell-recovery/os_partition')
+                os_part,swap_part = self.dual_partition_num(os_label)
+        except debconf.DebconfError as err:
+            self.log(str(err))
+
         if self.test_swap():
             swap_part = ''
-        #remove extras
-        for number in (EFI_OS_PARTITION, swap_part):
+       #remove extras
+        for number in (os_part,swap_part):
             if number.isdigit():
                 remove = misc.execute_root('parted', '-s', self.device, 'rm', number)
                 if remove is False:
@@ -429,6 +455,27 @@ class Page(Plugin):
                 if refresh is False:
                     self.log("Error updating partition %s for kernel device %s (this may be normal)'" % (number, self.device))
 
+    def dual_partition_num(self,label):
+       #remove UBUNTU patition for dual boot
+       ##OS num
+        os_part = ''
+        swap_part = ''
+        digits = re.compile('\d+')
+        try:
+            os_path = magic.fetch_output(['readlink','/dev/disk/by-label/'+label]).split('\n')
+        except Exception as err:
+            self.log('os_path command is executed failed, the error is %s'%str(err))
+
+        os_part = digits.search(os_path[0].split('/')[-1]).group()
+
+        with misc.raised_privileges():
+            partitions = magic.fetch_output(['parted','-s',self.device,'print']).split('\n')
+            for line in partitions:
+                if 'linux-swap' in line:
+                    swap_part = line.split()[0]
+
+        return os_part,swap_part
+
     def explode_sdr(self):
         '''Explodes all content explicitly defined in an SDR
            If no SDR was found, don't change drive at all
@@ -553,7 +600,7 @@ class Page(Plugin):
             drive_obj = block.get_cached_property("Drive").get_string()
             if drive_obj == '/':
                 continue
-                
+
             drive = udisks.get_object(drive_obj).get_drive()
             if drive:
                 bus = drive.get_cached_property("ConnectionBus").get_string()
@@ -563,11 +610,9 @@ class Page(Plugin):
                     media = drive.get_cached_property("Media").get_string()
                     if not media:
                         continue
-                elif drive.get_cached_property("Removable").get_boolean():
-                    continue
             else:
                 continue
-        
+
             devicesize = drive.get_cached_property("Size").unpack()
             if devicesize == 0:
                 continue
@@ -576,18 +621,35 @@ class Page(Plugin):
             devicemodel = drive.get_cached_property("Model").get_string()
             devicevendor = drive.get_cached_property("Vendor").get_string()
             devicesize_gb = "%i" % (devicesize / 1000000000)
-            
+
             disks.append([devicefile, devicesize, "%s GB %s %s (%s)" % (devicesize_gb, devicevendor, devicemodel, devicefile)])
 
-        #If multiple candidates were found, record in the logs
         if len(disks) == 0:
             raise RuntimeError("Unable to find and candidate hard disks to install to.")
-        if len(disks) > 1:
+
+        # Search for the recovery partition on the same disk first
+        the_same_disk = None
+        for disk in disks:
+            device = disk[0]
+            with open('/proc/mounts', 'r') as mounts:
+                for line in mounts.readlines():
+                    if device in line:
+                        self.device = device
+                        the_same_disk = disk
+                        break
+            if the_same_disk:
+                break
+        if the_same_disk:
+            disks.remove(the_same_disk)
+            disks.insert(0, the_same_disk)
+        else:
             disks.sort()
+            self.device = disks[0][0]
+
+        #If multiple candidates were found, record in the logs
+        if len(disks) > 1:
             self.log("Multiple disk candidates were found: %s" % disks)
 
-        #Always choose the first candidate to start
-        self.device = disks[0][0]
         self.log("Initially selected candidate disk: %s" % self.device)
 
         #populate UI
@@ -631,7 +693,7 @@ class Page(Plugin):
         with misc.raised_privileges():
             version = magic.check_version()
         self.log("version %s" % version)
-        
+
         #mountpoint
         mount = find_boot_device()
         self.log("mounted from %s" % mount)
@@ -691,7 +753,7 @@ class Page(Plugin):
             proprietary = ''
 
         #If we detect that we are booted into uEFI mode, then we only want
-        #to do a GPT install.  
+        #to do a GPT install.
         if os.path.isdir('/proc/efi') or os.path.isdir('/sys/firmware/efi'):
             self.efi = True
 
@@ -745,7 +807,7 @@ class Page(Plugin):
         client_type = os.path.join('/cdrom', '.oem', 'client_type')
         if os.path.isfile(client_type):
             with open (client_type, "r") as myfile:
-                content=myfile.read().replace('\n', '') 
+                content=myfile.read().replace('\n', '')
             if content == "kylin":
                 language = 'zh_CN.UTF-8'
             self.preseed('debian-installer/locale', language)
@@ -754,34 +816,27 @@ class Page(Plugin):
         #Clarify which device we're operating on initially in the UI
         try:
             self.fixup_recovery_devices()
+            self.log("rec_type %s, stage %d, device %s" % (rec_type, self.stage, self.device))
             if (rec_type == 'factory' and self.stage == 2) or rec_type == 'hdd':
                 self.fixup_factory_devices(rec_part)
         except Exception as err:
             self.handle_exception(err)
             self.cancel_handler()
 
-        # Always put 'ubuntu' in BootNext if any
-        with misc.raised_privileges():
-            bootmgr_output = magic.fetch_output(['efibootmgr', '-v']).split('\n')
-            for line in bootmgr_output:
-                bootnum = ''
-                if line.startswith('Boot') and 'ubuntu' in line:
-                    bootnum = line.split('Boot')[1].replace('*', '').split()[0]
-                    misc.execute_root('efibootmgr', '-n', bootnum)
-
         return (['/usr/share/ubiquity/dell-bootstrap'], [RECOVERY_TYPE_QUESTION])
 
     def ok_handler(self):
         """Copy answers from debconf questions"""
         #basic questions
         rec_type = self.ui.get_type()
-        self.log("recovery type set to %s" % rec_type)
+        self.log("recovery type set to '%s'" % rec_type)
         self.preseed(RECOVERY_TYPE_QUESTION, rec_type)
         (device, size) = self.ui.get_selected_device()
         if device:
             self.device = device
         if size:
             self.device_size = size
+        self.log("selected device %s %d" % (device, size))
 
         #advanced questions
         for question in [SWAP_QUESTION,
@@ -831,7 +886,7 @@ class Page(Plugin):
                                                "0")
                 size_thread.progress = self.report_progress
                 #init builder
-                self.rp_builder = RPbuilder(self.device, 
+                self.rp_builder = RPbuilder(self.device,
                                             self.device_size,
                                             self.rp_filesystem,
                                             self.mem,
@@ -875,6 +930,12 @@ class Page(Plugin):
 
     def cancel_handler(self):
         """Called when we don't want to perform recovery'"""
+        if os.path.exists(os.path.join('/cdrom', '.disk', 'info.recovery')) and \
+           os.path.exists(os.path.join('/cdrom', '.disk', 'info')) and \
+           misc.execute_root('mount', '-o', 'remount,rw', '/cdrom'):
+            with misc.raised_privileges():
+                os.remove(os.path.join('/cdrom', '.disk', 'info'))
+            misc.execute_root('mount', '-o', 'remount,ro', '/cdrom')
         misc.execute_root('reboot')
 
     def handle_exception(self, err):
@@ -937,7 +998,7 @@ manually to proceed.")
             raise RuntimeError("Error creating new partition table on %s" % (self.device))
 
         self.status("Creating Partitions", 1)
-        grub_size = 50
+        grub_size = 250
         commands = [('parted', '-a', 'optimal', '-s', self.device, 'mkpart', 'primary', 'fat16', '0', str(grub_size)),
                     ('parted', '-s', self.device, 'name', '1', "'EFI System Partition'"),
                     ('parted', '-s', self.device, 'set', '1', 'boot', 'on')]
@@ -950,6 +1011,10 @@ manually to proceed.")
             rp_part = EFI_RP_PARTITION
             esp_part = EFI_ESP_PARTITION
         for command in commands:
+            #wait for settle
+            if command[0] == 'mkfs.msdos':
+                while not os.path.exists(command[-1]):
+                    time.sleep(1)
             result = misc.execute_root(*command)
             if result is False:
                 if self.efi:
@@ -967,6 +1032,8 @@ manually to proceed.")
             command = ('mkfs.msdos', '-n', 'OS', self.device + rp_part)
         elif self.rp_type == 'ntfs':
             command = ('mkfs.ntfs', '-f', '-L', 'RECOVERY', self.device + rp_part)
+        while not os.path.exists(command[-1]):
+            time.sleep(1)
         result = misc.execute_root(*command)
         if result is False:
             raise RuntimeError("Error creating %s filesystem on %s%s" % (self.rp_type, self.device, rp_part))
@@ -1016,7 +1083,7 @@ manually to proceed.")
 
         #Check for a grub.cfg - replace as necessary
         files = {'recovery_partition.cfg': 'grub.cfg',
-                 'common.cfg' : 'common.cfg'} 
+                }
         for item in files:
             full_path = os.path.join('/mnt', 'factory', files[item])
             if os.path.exists(full_path):
@@ -1055,10 +1122,10 @@ manually to proceed.")
             #delete old entries
             for line in bootmgr_output:
                 bootnum = ''
-                if line.startswith('Boot') and 'ubuntu' in line:
+                if line.startswith('Boot') and 'ubuntu' in line.lower():
                     bootnum = line.split('Boot')[1].replace('*', '').split()[0]
                 if bootnum:
-                    bootmgr = misc.execute_root('efibootmgr', '-q', '-b', bootnum, '-B')
+                    bootmgr = misc.execute_root('efibootmgr', '-v', '-b', bootnum, '-B')
                     if bootmgr is False:
                         raise RuntimeError("Error removing old EFI boot manager entries")
 
@@ -1067,19 +1134,10 @@ manually to proceed.")
             os.rename(os.path.join(direct_path, 'bootx64.efi'),
                       os.path.join(direct_path, target))
 
-        add = misc.execute_root('efibootmgr', '-c', '-d', self.device, '-p', EFI_ESP_PARTITION, '-l', '\\EFI\\ubuntu\\%s' % target, '-L', 'ubuntu')
+        add = misc.execute_root('efibootmgr', '-v', '-c', '-d', self.device, '-p', EFI_ESP_PARTITION, '-l', '\\EFI\\ubuntu\\%s' % target, '-L', 'ubuntu')
         if add is False:
             raise RuntimeError("Error adding efi entry to %s%s" % (self.device, esp_part))
 
-        #find new entry and put it in BootNext
-        with misc.raised_privileges():
-            bootmgr_output = magic.fetch_output(['efibootmgr', '-v']).split('\n')
-            for line in bootmgr_output:
-                bootnum = ''
-                if line.startswith('Boot') and 'ubuntu' in line:
-                    bootnum = line.split('Boot')[1].replace('*', '').split()[0]
-                    misc.execute_root('efibootmgr', '-n', bootnum)
-
         ##clean up ESP mount
         misc.execute_root('umount', '/mnt/efi')
 
@@ -1194,7 +1252,7 @@ def find_n_set_iterator(combobox, value, column = 0):
 ###########################################
 class Install(InstallPlugin):
     """The install time dell-bootstrap ubiquity plugin"""
-    
+
     def __init__(self, frontend, db=None, ui=None):
         self.progress = None
         self.target = None
@@ -1208,43 +1266,6 @@ class Install(InstallPlugin):
             if line.startswith('ricoh_mmc'):
                 misc.execute('rmmod', line.split()[0])
 
-    def enable_oem_config(self):
-        '''Enables OEM config on the target'''
-        oem_dir = os.path.join(self.target, 'var/lib/oem-config')
-        if not os.path.exists(oem_dir):
-            os.makedirs(oem_dir)
-        with open(os.path.join(oem_dir, 'run'), 'w'):
-            pass
-
-    def propagate_kernel_parameters(self):
-        '''Copies in kernel command line parameters that were needed during
-           installation'''
-        extra = magic.find_extra_kernel_options()
-        new = ''
-        for item in extra.split():
-            if not 'debian-installer/'                in item and \
-               not 'console-setup/'                   in item and \
-               not 'locale='                          in item and \
-               not 'BOOT_IMAGE='                      in item and \
-               not 'iso-scan/'                        in item and \
-               not 'ubiquity'                         in item:
-                new += '%s ' % item
-        extra = new.strip()
-
-        grubf = os.path.join(self.target, 'etc/default/grub')
-        if extra and os.path.exists(grubf):
-            #read/write new grub
-            with open(grubf, 'r') as rfd:
-                default_grub = rfd.readlines()
-            with open(grubf, 'w') as wfd:
-                for line in default_grub:
-                    if 'GRUB_CMDLINE_LINUX_DEFAULT' in line:
-                        line = line.replace('GRUB_CMDLINE_LINUX_DEFAULT="', \
-                                      'GRUB_CMDLINE_LINUX_DEFAULT="%s ' % extra)
-                    wfd.write(line)
-            from ubiquity import install_misc
-            install_misc.chrex(self.target, 'update-grub')
-
     def remove_unwanted_drivers(self):
         '''Removes drivers that were preseeded to not used for postinstall'''
         drivers = ''
@@ -1289,13 +1310,6 @@ class Install(InstallPlugin):
         to_install = []
         to_remove  = []
 
-        #Determine if we are doing OOBE
-        try:
-            if progress.get('oem-config/enable') == 'true':
-                self.enable_oem_config()
-        except debconf.DebconfError:
-            pass
-
         #if we are loop mounted, make sure the chroot knows it too
         if os.path.isdir(magic.ISO_MOUNT):
             os.makedirs(os.path.join(self.target, magic.ISO_MOUNT.lstrip('/')))
@@ -1344,10 +1358,8 @@ class Install(InstallPlugin):
         to_install += magic.mark_upgrades()
 
         self.remove_unwanted_drivers()
-                    
-        self.remove_ricoh_mmc()
 
-        self.propagate_kernel_parameters()
+        self.remove_ricoh_mmc()
 
         self.wake_network()
 
@@ -1355,4 +1367,3 @@ class Install(InstallPlugin):
         install_misc.record_removed(to_remove)
 
         return InstallPlugin.install(self, target, progress, *args, **kwargs)
-
